/*
 * Copyright(C) OASIS(R) 2006,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */

/** @file
 * This file contains the SCA RefCountingPointer class declaration
 * and cast operations on RefCountingPointers.
 * @author SCA-C-C++ TC
 * @version 1.1
 * @date 4 March 2010
 */

#ifndef _oasis_sca_RefCountingPointer_h_
#define _oasis_sca_RefCountingPointer_h_

#include <cstddef>

namespace oasis {
    namespace sca {

        template <typename T>

        /**
         * A smart pointer class that holds a real (dumb) pointer to the object.
         * If the reference counting pointer is copied, then a duplicate pointer is returned with the same real pointer.
         * A reference count within the object is incremented for each copy of the pointer, so only when all pointers
         * go out of scope will the object be freed.
         */
        class RefCountingPointer {
            public:
            	/**
                 * Dereferences the underlying pointer
                 * @return A reference to the value of the pointer.
                 * @throws SCANullPointerException if the pointer is NULL
                 */
                T& operator* () const{
                	return *ptr_;
                };


                /**
                 * Used to invoke member functions on the underlying pointer. This is equivalent to invoking p->func()
                 * where func() is a member function defined on the underlying pointer type.
                 * @throws SCANullPointerException if the pointer is NULL
                 */
                T* operator-> () const{
                	return 0;
                };


                /**
                 * Used to determine if the underlying pointer is set, i.e. if (p)
                 * @return Zero if the underlying pointer is null, otherwise a non-zero value.
                 */
                operator void* () const{
                	return 0;
                };


                /**
                 * Used to determine if the underlying pointer is not set, i.e. if (!p)
                 * @return A non-zero value if the underlying pointer is null, otherwise zero.
                 */
                bool operator! () const{
                	return 0;
                };

                RefCountingPointer(T* ptr) : ptr_(ptr) {};
                RefCountingPointer() {};

                template <typename U>
                RefCountingPointer(RefCountingPointer<U> ptr) : ptr_(dynamic_cast<T*>(ptr.ptr_)) {};

                T* ptr_;

        };


        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the const_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         */
        RefCountingPointer<T> constCast(RefCountingPointer<U> other){
        	return NULL;
        };



        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the dynamic_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         *         This may return an invalid RefCountingPointer instance if the cast fails.
         */
        RefCountingPointer<T> dynamicCast(RefCountingPointer<U> other){
        	return RefCountingPointer<T>(dynamic_cast<T*>(other.ptr_));
        };



        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the reinterpret_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         */
        RefCountingPointer<T> reinterpretCast(RefCountingPointer<U> other){
        	return NULL;
        };



        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the static_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         */
        RefCountingPointer<T> staticCast(RefCountingPointer<U> other){
        	return NULL;
        };



    } // namespace sca
} // namespace oasis

#endif /* _oasis_sca_RefCountingPointer_h_ */
